#include"gba.h"
#include"data.h"
#include"sprite.h"
#include"animation.h"

#define fixed s32

#define UP 1
#define LEFT 2
#define DOWN 3
#define RIGHT 0
#define MAX_TYPES 2
static u16 offset[] = {0,64,80};
u8 OAMTracker[128];  //keeps track of used sprites
u8 ROTTracker[32];	 //keeps track of rotation / scale perameters used

extern fixed COS[360];
extern fixed SIN[360];

extern u16 bg0hofs;
extern char rand(void);
extern void DMA3Call(u32* Src,u32* Dst,u32 Cnt);
extern u32* OAMdata;


#define SetBit(reg,n) (reg) |= (1<<(n))
#define ClearBit(reg,n) (reg) &= (~(1<<(n)))

extern u16 OAM[4*128];



/////////////Function Definitions////////////////
void S_Move(pSprite sp, u16 x, u16 y)
{
	
	
	sp->Atribute1 &= ~0x00FF;
	sp->Atribute2 &= ~0x01ff;

	sp->Atribute1 |= (u8)y;
	sp->Atribute2 |= (u8)x;

	OAM[sp->SpriteNumber] = sp->Atribute1;
	OAM[sp->SpriteNumber+1] = sp->Atribute2;

}

void S_Kill(pSprite sp)
{
	S_Move(sp,241,241);  //move sprite offscreen
	OAMTracker[sp->SpriteNumber/4] = 0;
	ROTTracker[sp->rotind] = 0;
}
void S_ChangeState(pSprite sp, u8 state)
{
	if(sp->State != state )
	{
	
		sp->cur_animation = sp->animations[state];
		sp->cur_frame = 0;
		sp->count = 0;
		sp->fDone = 0;
		sp->State = state;
	}
	
}
void S_Animate(pSprite sp)
{	
	Animate* a;
	
//	u16 oldx;
//	oldx = sp->x;
	a = &sp->cur_animation;

	if((sp->count > a->duration))// != S_STANDING)
	{
		sp->count = 0;
		
		
		sp->frame = sp->frames[sp->cur_frame];
		
		sp->cur_frame++;
		
	//	DMA3Call((u32*)&sp->data[16*4*sp->frame + a->offset ],(u32*)sp->offset,0x84000100);
	}//end if

	sp->count++;
	if(sp->cur_frame >= a->num_frames)
	{
		sp->fDone = 1;
		sp->cur_frame = 0;
		S_ChangeState(sp,S_STANDING);
	}
	
	


		
}
void S_CopyToRam(pSprite sp)
{
	
}
int S_Initialize(pSprite sp,  u16 x, u16 y,char shape, char type)
{
	int i;
	
	sp->direction = 0;
	sp->x = x;
	sp->y = y;
	sp->type = type;
	sp->Atribute1 = y;
	sp->Atribute2 = x;
	sp->Atribute3 = offset[type];
	
	sp->Atribute1 |= 0x2000;//SetBit(sp->Atribute1,13);
	switch(shape)
	{
	case SIZE_64x64:
		{
			SetBit(sp->Atribute2,14);
			SetBit(sp->Atribute2,15);
		}break;
	case SIZE_32x32:
		{
			SetBit(sp->Atribute2,15);
			
		}break;
	case SIZE_16x16:
		{
			SetBit(sp->Atribute2,14);
		}break;
	default: break;
	}

	switch(type)
	{
	case 0:
		{
			
			
			sp->motion = (s16*)guy_motion;
			sp->frames = (u8*)guy_frames;
			sp->offset = OAMdata;
			sp->data = (u32*)tguy;
		}break;
	
	default: break;
	}

	i = 0;
	while(OAMTracker[i])//gets the next OAM number that is empty
		i++;
	OAMTracker[i] = 1;

	sp->SpriteNumber = i*4;
	
	OAM[sp->SpriteNumber] = sp->Atribute1;
	OAM[sp->SpriteNumber+1] = sp->Atribute2;
	OAM[sp->SpriteNumber+2] = sp->Atribute3;
	sp->rotind = -1;
	sp->direction = RIGHT;
	sp->frame = 0;
	sp->cur_animation = sp->animations[S_STANDING];
	sp->State = S_STANDING;
	return 0;
}

void S_Flip(pSprite sp, u8 fV, u8 fH)
{
	ClearBit(sp->Atribute2,13);
	ClearBit(sp->Atribute2,12);
	ClearBit(sp->Atribute1,8);
	if(fV)
		SetBit(sp->Atribute2,13);
	if(fH)
		SetBit(sp->Atribute2,12);
	OAM[sp->SpriteNumber] = sp->Atribute1;
	OAM[sp->SpriteNumber+1] = sp->Atribute2;
	
}
void S_Rotate(pSprite sp, u16 angle)
{
	int i;
	if(sp->rotind < 0)  //this just keeps track of wich rotation indexes are used
	{					//rotind is initialized to -1
		i=0;
		while(ROTTracker[i])  //just an aray to keep track of used pa,pb,pc,pd
			i++;
		sp->rotind = i;
		ROTTracker[i] = 1;
	}
	sp->Atribute2 &= 0xC1FF;  //clear the bits for rotation index
	sp->Atribute2 |= sp->rotind<<9;  //set the index
	SetBit(sp->Atribute1,8);  //allow rotation
	OAM[sp->SpriteNumber] = sp->Atribute1;  //put it in oam
	OAM[sp->SpriteNumber + 1] = sp->Atribute2; 
	
	OAM[(sp->rotind*16)+3] = COS[angle];  //cos&sin are LUTs that are .8 fixed numbers
	OAM[(sp->rotind*16)+7] = SIN[angle];
	OAM[(sp->rotind*16)+11] = -SIN[angle];
	OAM[(sp->rotind*16)+15] = COS[angle];

	
}
